  Vec = {}

function Vec.sum (a,b)
      if getmetatable(a) ~= Vec.mt or getmetatable(b) ~= Vec.mt then
        error("attempt to `add' a vector with a non-vector value", 2)
      end
      local res = Vector3d(a.x+b.x,a.y+b.y,a.z+b.z)
      return res
end


function Vec.equal (a,b)
      if getmetatable(a) ~= Vec.mt or getmetatable(b) ~= Vec.mt then
        error("attempt to `compare' a vector with a non-vector value", 2)
      end
return (a.x==b.x and a.y==b.y and a.z==b.z)
end

function Vec.sub(a,b)
      if getmetatable(a) ~= Vec.mt or getmetatable(b) ~= Vec.mt then
        error("attempt to `subtract' a vector with a non-vector value", 2)
      end
      local res = Vector3d(a.x-b.x,a.y-b.y,a.z-b.z)
      return res
end

function Vec.unm(a)
	local res = Vector3d(-a.x,-a.y,-a.z)
       return res
end

function Vec.tostring(a)
	return "("..a.x..","..a.y..","..a.z..")"
end

function Vec.multiply(a,b)  --at least one must be a vector
	if tonumber(a) then --scalar
		return vec(a*b.x,a*b.y,a*b.z)
	elseif tonumber(b) then --scalar
		return vec(b*a.x,b*a.y,b*a.z)
	elseif type(a) == "vector" and type(b) == "vector" then			--dot product
		return a.x*b.x+a.y*b.y+a.z*b.z	--returns a scalar
	elseif type(a) == "vector" and type(b) == "matrix" then
		return Mat.multiply(a,b)
	end
end

Vec.mt = {}    -- metatable for sets

--[[function Vec.new (t)   -- 2nd version
      local set = {}
      setmetatable(set, Vec.mt)
      --for _, l in ipairs(t) do set[l] = true end
      return set
end]]

function Vec:magnitude()
	return math.sqrt(self.x^2 + self.y^2 + self.z^2)
end

function Vec:sqrmagnitude()
	return self.x^2 + self.y^2 + self.z^2
end

function Vec:normalize()
	return (1/self:magnitude())*self
end

function Vec.cross(a,b)
	return vec(a.y*b.z-a.z*b.y,a.z*b.x-a.x*b.z,a.x*b.y-a.y*b.x)
end

 Vec.mt.__add = Vec.sum
 Vec.mt.__eq = Vec.equal
 Vec.mt.__index = Vec
 Vec.mt.__sub = Vec.sub
 Vec.mt.__unm = Vec.unm
 Vec.mt.__mul = Vec.multiply
 Vec.mt.__tostring = Vec.tostring
 Vec.mt.__pow = Vec.cross  --in the lack of a better symbol


function Vector3d(x,y,z)
	local vec = {x=x,y=y,z=z,type="vector"}
	setmetatable(vec, Vec.mt)
	return vec
end

vec = Vector3d